package javax.rmi.CORBA;

import java.net.MalformedURLException;
import java.rmi.server.RMIClassLoader;
import java.util.Properties;
import com.sun.corba.se.impl.javax.rmi.CORBA.StubDelegateImpl;
import org.omg.CORBA.INITIALIZE;
import java.security.AccessController;
import com.sun.corba.se.impl.orbutil.GetPropertyAction;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.rmi.RemoteException;
import org.omg.CORBA.ORB;
import java.io.Serializable;
import org.omg.CORBA_2_3.portable.ObjectImpl;

public abstract class Stub extends ObjectImpl implements Serializable
{
    private static final long serialVersionUID = 1087775603798577179L;
    private transient StubDelegate stubDelegate;
    private static Class stubDelegateClass;
    private static final String StubClassKey = "javax.rmi.CORBA.StubClass";
    
    public Stub() {
        this.stubDelegate = null;
    }
    
    public int hashCode() {
        if (this.stubDelegate == null) {
            this.setDefaultDelegate();
        }
        if (this.stubDelegate != null) {
            return this.stubDelegate.hashCode(this);
        }
        return 0;
    }
    
    public boolean equals(Object o) {
        if (this.stubDelegate == null) {
            this.setDefaultDelegate();
        }
        return this.stubDelegate != null && this.stubDelegate.equals(this, o);
    }
    
    public String toString() {
        if (this.stubDelegate == null) {
            this.setDefaultDelegate();
        }
        if (this.stubDelegate != null) {
            String string = this.stubDelegate.toString(this);
            if (string == null) {
                return super.toString();
            }
            return string;
        }
        else {
            return super.toString();
        }
    }
    
    public void connect(ORB orb) throws RemoteException {
        if (this.stubDelegate == null) {
            this.setDefaultDelegate();
        }
        if (this.stubDelegate != null) {
            this.stubDelegate.connect(this, orb);
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        if (this.stubDelegate == null) {
            this.setDefaultDelegate();
        }
        if (this.stubDelegate != null) {
            this.stubDelegate.readObject(this, objectInputStream);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (this.stubDelegate == null) {
            this.setDefaultDelegate();
        }
        if (this.stubDelegate != null) {
            this.stubDelegate.writeObject(this, objectOutputStream);
        }
    }
    
    private void setDefaultDelegate() {
        if (Stub.stubDelegateClass != null) {
            try {
                this.stubDelegate = (StubDelegate)Stub.stubDelegateClass.newInstance();
            }
            catch (Exception ex) {}
        }
    }
    
    private static Object createDelegate(String key) {
        String property = (String)AccessController.doPrivileged(new GetPropertyAction(key));
        if (property == null) {
            Properties orbPropertiesFile = getORBPropertiesFile();
            if (orbPropertiesFile != null) {
                property = orbPropertiesFile.getProperty(key);
            }
        }
        if (property == null) {
            return new StubDelegateImpl();
        }
        try {
            return loadDelegateClass(property).newInstance();
        }
        catch (ClassNotFoundException ex) {
            INITIALIZE o = new INITIALIZE("Cannot instantiate " + property);
            o.initCause(ex);
            throw o;
        }
        catch (Exception ex2) {
            INITIALIZE o2 = new INITIALIZE("Error while instantiating" + property);
            o2.initCause(ex2);
            throw o2;
        }
    }
    
    private static Class loadDelegateClass(String str) throws ClassNotFoundException {
        try {
            return Class.forName(str, false, Thread.currentThread().getContextClassLoader());
        }
        catch (ClassNotFoundException ex) {
            try {
                return RMIClassLoader.loadClass(str);
            }
            catch (MalformedURLException ex2) {
                throw new ClassNotFoundException("Could not load " + str + ": " + ex2.toString());
            }
        }
    }
    
    private static Properties getORBPropertiesFile() {
        return (Properties)AccessController.doPrivileged(new GetORBPropertiesFileAction());
    }
    
    static {
        Stub.stubDelegateClass = null;
        Object delegate = createDelegate("javax.rmi.CORBA.StubClass");
        if (delegate != null) {
            Stub.stubDelegateClass = delegate.getClass();
        }
    }
}
